summary refs log tree commit diff
diff options
context:
space:
mode:
authorEric Lapuyade <eric.lapuyade@linux.intel.com>2013-09-23 17:56:31 +0200
committerSamuel Ortiz <sameo@linux.intel.com>2013-09-25 14:25:41 +0200
commit22d4aae5897fb8355130b8f7d9a3af153eac9714 (patch)
treeb11eb743ff1f9a816875b43237ae51f28695b421
parenta4ada6cadb8a2246f263ff6a0d0cca8832f3970e (diff)
downloadlinux-22d4aae5897fb8355130b8f7d9a3af153eac9714.tar.gz
NFC: NCI: nci_spi_recv_frame() now returns (not forward) the read frame
Previously, nci_spi_recv_frame() would directly transmit incoming frames
to the NCI Core. However, it turns out that some NFC NCI Chips will add
additional proprietary headers that must be handled/removed before NCI
Core gets a chance to handle the frame. With this modification, the chip
phy or driver are now responsible to transmit incoming frames to NCI
Core after proper treatment, and NCI SPI becomes a driver helper instead
of sitting between the NFC driver and NCI Core.

As a general rule in NFC, *_recv_frame() APIs are used to deliver an
incoming frame to an upper layer. To better suit the actual purpose of
nci_spi_recv_frame(), and go along with its nci_spi_send()
counterpart, the function is renamed to nci_spi_read()

The skb is returned as the function result

Signed-off-by: Eric Lapuyade <eric.lapuyade@intel.com>
Signed-off-by: Samuel Ortiz <sameo@linux.intel.com>
-rw-r--r--include/net/nfc/nci_core.h2
-rw-r--r--net/nfc/nci/spi.c26
2 files changed, 11 insertions, 17 deletions
diff --git a/include/net/nfc/nci_core.h b/include/net/nfc/nci_core.h
index 37ba06f2dfa9..1d505317dc67 100644
--- a/include/net/nfc/nci_core.h
+++ b/include/net/nfc/nci_core.h
@@ -233,6 +233,6 @@ struct nci_spi *nci_spi_allocate_spi(struct spi_device *spi,
 				     u8 acknowledge_mode, unsigned int delay,
 				     struct nci_dev *ndev);
 int nci_spi_send(struct nci_spi *nspi, struct sk_buff *skb);
-int nci_spi_recv_frame(struct nci_spi *nspi);
+struct sk_buff *nci_spi_read(struct nci_spi *nspi);
 
 #endif /* __NCI_CORE_H */
diff --git a/net/nfc/nci/spi.c b/net/nfc/nci/spi.c
index c111506b36d1..734c6dde7751 100644
--- a/net/nfc/nci/spi.c
+++ b/net/nfc/nci/spi.c
@@ -21,7 +21,6 @@
 #include <linux/export.h>
 #include <linux/spi/spi.h>
 #include <linux/crc-ccitt.h>
-#include <linux/nfc.h>
 #include <net/nfc/nci_core.h>
 
 #define NCI_SPI_ACK_SHIFT		6
@@ -164,7 +163,7 @@ static int send_acknowledge(struct nci_spi *nspi, u8 acknowledge)
 	return ret;
 }
 
-static struct sk_buff *__nci_spi_recv_frame(struct nci_spi *nspi)
+static struct sk_buff *__nci_spi_read(struct nci_spi *nspi)
 {
 	struct sk_buff *skb;
 	struct spi_message m;
@@ -258,7 +257,7 @@ static u8 nci_spi_get_ack(struct sk_buff *skb)
 }
 
 /**
  * nci_spi_recv_frame - receive frame from NCI SPI drivers
  *
  * @nspi: The nci spi
  * Context: can sleep
@@ -266,21 +265,18 @@ static u8 nci_spi_get_ack(struct sk_buff *skb)
  * This call may only be used from a context that may sleep.  The sleep
  * is non-interruptible, and has no timeout.
  *
- * It returns zero on success, else a negative error code.
+ * It returns an allocated skb containing the frame on success, or NULL.
  */
-int nci_spi_recv_frame(struct nci_spi *nspi)
+struct sk_buff *nci_spi_read(struct nci_spi *nspi)
 {
 	struct sk_buff *skb;
-	int ret = 0;
 
 	nspi->ops->deassert_int(nspi);
 
 	/* Retrieve frame from SPI */
-	skb = __nci_spi_recv_frame(nspi);
-	if (!skb) {
-		ret = -EIO;
+	skb = __nci_spi_read(nspi);
+	if (!skb)
 		goto done;
-	}
 
 	if (nspi->acknowledge_mode == NCI_SPI_CRC_ENABLED) {
 		if (!nci_spi_check_crc(skb)) {
@@ -299,20 +295,18 @@ int nci_spi_recv_frame(struct nci_spi *nspi)
 	/* If there is no payload (ACK/NACK only frame),
 	 * free the socket buffer
 	 */
-	if (skb->len == 0) {
+	if (!skb->len) {
 		kfree_skb(skb);
+		skb = NULL;
 		goto done;
 	}
 
 	if (nspi->acknowledge_mode == NCI_SPI_CRC_ENABLED)
 		send_acknowledge(nspi, ACKNOWLEDGE_ACK);
 
-	/* Forward skb to NCI core layer */
-	ret = nci_recv_frame(nspi->ndev, skb);
-
 done:
 	nspi->ops->assert_int(nspi);
 
-	return ret;
+	return skb;
 }
-EXPORT_SYMBOL_GPL(nci_spi_recv_frame);
+EXPORT_SYMBOL_GPL(nci_spi_read);
@@ -258,7 +257,7 @@ static u8 nci_spi_get_ack(struct sk_buff *skb)
 
 /**
- * nci_spi_recv_frame - receive frame from NCI SPI drivers
+ * nci_spi_read - read frame from NCI SPI drivers
  *
  * @nspi: The nci spi